<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Marco Gilbert <marco.gilbert@gmx.net>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *
 *
 *   59: class tx_dsbdwzlist_pi1 extends tslib_pibase
 *   74:     function main($content, $conf)
 *  105:     protected function init()
 *  132:     protected function fetchConfigurationValue($param)
 *  196:     protected function getPlotJava($turnierdata)
 *  230:     private function convertToRendercharset(&$value, $key, $param)
 *  240:     protected function getSingleViewData($pkz)
 *  269:     protected function parseTurniercode($turniercode)
 *  293:     protected function singleView($pkz)
 *  404:     protected function getLastEvaluation($date)
 *  414:     protected function getListViewData($zps)
 *  452:     protected function listView($zps)
 *
 * TOTAL FUNCTIONS: 13
 * (This index is automatically created/updated by the extension "extdeveval")
 *
 */

if (!class_exists('tslib_pibase')) { require_once(PATH_tslib . 'class.tslib_pibase.php'); }


/**
 * Plugin 'DWZ List' for the 'dsb_dwzlist' extension.
 *
 * @author	Marco Gilbert <marco.gilbert@googlemail.com>
 * @package	TYPO3
 * @subpackage	tx_dsbdwzlist
 */
class tx_dsbdwzlist_pi1 extends tslib_pibase {
	public $prefixId = 'tx_dsbdwzlist_pi1'; // Same as class name
	public $scriptRelPath = 'pi1/class.tx_dsbdwzlist_pi1.php'; // Path to this script relative to the extension dir.
	public $extKey = 'dsb_dwzlist'; // The extension key.
	public $pi_checkCHash = true;
	private $templateCode;
	private $allowCaching = true;

	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	string		The content that is displayed on the website
	 */
	function main($content, $conf) {
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();

		// Check environment
		if (!isset($conf['zps'])) {
			return $this->pi_wrapInBaseClass($this->pi_getLL('no_ts_template'));
		}

		// Init
		$this->init();

		if(isset($this->piVars['pkz'])) {
			// singleView
			$pkz = $this->piVars['pkz'];
			$content = $this->singleView($pkz);
		}
		else {
			// listView
			$content = $this->listView($this->conf['zps']);
		}

		return $this->pi_wrapInBaseClass($content);
	}

	/**
	 * Initializes plugin configuration.
	 *
	 * @return	string		Generated HTML
	 */
	protected function init() {
		// Init Flexform configuration for plugin
		$this->pi_initPIflexForm();

		// Get values
		$this->conf['zps'] = $this->fetchConfigurationValue('zps');
		$this->conf['templateFile'] = $this->fetchConfigurationValue('templateFile');

		// Set defaults if necessary
		if (!$this->conf['zps']) {
			$this->conf['zps'] = 53006; // My Home Club
		}
		if (!$this->conf['templateFile']) {
			$this->conf['templateFile'] = 'EXT:' . $this->extKey . '/res/pi1_template.html';
		}

		// Load template code
		$this->templateCode = $this->cObj->fileResource($this->conf['templateFile']);
	}


	/**
	 * Fetches configuration value given its name. Merges flexform and TS configuration values.
	 *
	 * @param	string		$param	Configuration value name
	 * @return	string		Parameter value
	 */
	protected function fetchConfigurationValue($param) {
		$value = trim($this->pi_getFFvalue($this->cObj->data['pi_flexform'], $param));
		return $value ? $value : $this->conf[$param];
	}


	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$plotdata: ...
	 * @return	[type]		...
	 */
	protected function getPlotJava($turnierdata) {
		$jqconfig = $this->conf['jqplot.'];

		$plot_dwz = array();
		$plot_lstng = array();

		foreach ($turnierdata as $line) {
			if($line['dwzneu'] !== '0' ) { $plot_dwz[] = array( intval($line['dwzneuindex']), intval($line['dwzneu'])); }
			if($line['leistung'] !== '0' ) { $plot_lstng[] = array( intval($line['dwzneuindex']), intval($line['leistung'])); }
		}


		$content = '$(document).ready(function () {';

		$content .= 'var line1=' . json_encode($plot_dwz) .  ';';
		$content .= 'var line2=' . json_encode($plot_lstng) .  ';';

		$content .= '$.jqplot("chartdiv", [line1,line2] ,';

		$value = new stdClass();

		$value->title = $jqconfig['config.']['title'];

		$value->axes = new stdClass();
		$value->axes->xaxis = new stdClass();
		$value->axes->xaxis->label = $jqconfig['config.']['axes.']['xaxis.']['label'];
		$value->axes->yaxis = new stdClass();
		$value->axes->yaxis->label = $jqconfig['config.']['axes.']['yaxis.']['label'];

		$value->seriesDefaults = new stdClass();
		$value->seriesDefaults->showMarker = (bool) $jqconfig['config.']['seriesDefaults.']['showMarker'];

		$value->series[0] = new stdClass();
		$value->series[0]->label = $jqconfig['config.']['series.']['10.']['label'];
		$value->series[0]->color = $jqconfig['config.']['series.']['10.']['color'];
		$value->series[0]->showMarker = (bool) $jqconfig['config.']['series.']['10.']['showMarker'];
		$value->series[0]->trendline = new stdClass();
		$value->series[0]->trendline->show = (bool) $jqconfig['config.']['series.']['10.']['trendline.']['show'];
		$value->series[0]->trendline->linePattern = $jqconfig['config.']['series.']['10.']['trendline.']['linePattern'];
		$value->series[0]->trendline->color = $jqconfig['config.']['series.']['10.']['trendline.']['color'];

		$value->series[1] = new stdClass();
		$value->series[1]->label = $jqconfig['config.']['series.']['20.']['label'];
		$value->series[1]->color = $jqconfig['config.']['series.']['20.']['color'];
		$value->series[1]->showMarker = (bool) $jqconfig['config.']['series.']['20.']['showMarker'];

		$value->legend = new stdClass();
		$value->legend->show = (bool) $jqconfig['config.']['legend.']['show'];
		$value->legend->location = $jqconfig['config.']['legend.']['location'];

		$value->highlighter = new stdClass();
		$value->highlighter->show = (bool) $jqconfig['config.']['highlighter.']['show'];

		$value->grid = new stdClass();
		$value->grid->background = $jqconfig['config.']['grid.']['background'];

		$content .= json_encode($value);

		$content .= ');});';


		return $content;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$value: ...
	 * @param	[type]		$key: ...
	 * @param	[type]		$param: ...
	 * @return	[type]		...
	 */
	private function convertToRendercharset(&$value, $key, $param) {
		$value = iconv('utf-8', $renderCharset, $value);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$pkz: ...
	 * @return	[type]		...
	 */
	protected function getSingleViewData($pkz) {

		$url = 'http://www.schachbund.de/php/dewis/spieler.php?pkz=' . htmlspecialchars($pkz) . '&format=array';

		// use cachefile. refresh every 60 minutes
		$cachefile = PATH_site . 'typo3temp/dsb_dwzlist/cache_' . $pkz;
		if(file_exists($cachefile) && (time() - filemtime($cachefile) < 60*60)){
			// read data from cachefile
			$urlcontent = t3lib_div::getURL($cachefile);
		} else {
			//retrieve data from url
			$urlcontent = t3lib_div::getURL($url);
			if($urlcontent){
				// if data successful fetched, write to file
				t3lib_div::writeFileToTypo3tempDir($cachefile, $urlcontent);
			} else if(file_exists($cachefile)){
				// fetching new data failed, use cached version.
				$urlcontent = t3lib_div::getURL($cachefile);
			}
		}

		// Daten als Array laden
		$array = unserialize($urlcontent);

		// determine fe render charset
		$renderCharset = $GLOBALS['TSFE']->renderCharset;

		// test if conversion from utf-8 charset is necessary
		if ($renderCharset != 'utf-8') {
			array_walk_recursive($array, 'tx_dsbdwzlist_pi1::convertToRendercharset', $renderCharset);
		}

		return $array;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$turniercode: ...
	 * @return	[type]		...
	 */
	protected function parseTurniercode($turniercode) {

		$fields = array('turniertermin', 'organisationsebene' , 'turnierart');
		$return = array_combine($fields, explode('-', $turniercode));

		$parts = str_split($return['turniertermin']);

		if (intval ( $parts[0] ) == 0) {
			$return['turnierjahr'] = 2000 + (ord(strtoupper($parts[0])) - 65) * 10 + intval($parts[1]);
		} else {
			$return['turnierjahr'] = 1900 + intval($parts[0]) * 10 + intval($parts[1]);
		}

		$return['turnierkw'] = intval($parts[2])*10 + intval($parts[3]);

		return $return;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$zps: ...
	 * @return	[type]		...
	 */
	protected function singleView($pkz) {

		$lines = $this->getSingleViewData($pkz);

		// Zeilennummern bei turnieren einfuegen
		foreach ($lines['turnier'] as $rowIndex => &$t) {
			$t['entrynr'] = $rowIndex+1;
			$tc = $this->parseTurniercode($t['turniercode']);
			$t['turnierjahr'] = $tc['turnierjahr'];
		}

		// import variables from an array into the current symbol table
		extract($lines);

		// Get template for SINGLE view
		$singleViewTemplate = $this->cObj->getSubpart($this->templateCode, '###SINGLE_VIEW###');
		$subTemplate['singlerow'] = $this->cObj->getSubpart($singleViewTemplate, '###SINGLEROW###');
		$subTemplate['row'] = $this->cObj->getSubpart($subTemplate['singlerow'], '###ITEM###');
		$subTemplate['row_alt'] = $this->cObj->getSubpart($subTemplate['singlerow'], '###ITEM_ALT###');

		// prepare all the necessary objects and arrays
		$local_cObj = t3lib_div::makeInstance('tslib_cObj');

		// fill singleview text markers array
		$markers = array('copyright', 'backlink', 'cardfrom', 'linknewwindow');
		foreach ($markers as $marker) {
			$m = 'singleviewtext_' . $marker;
			$markContentArray['###' . strtoupper($m) . '###'] = $local_cObj->stdWrap($this->pi_getLL(strtoupper($m)), $this->conf['singleView.'][$m . '_stdWrap.']);
		}

		// link zur alten karteikarte
		// Configure typolink functionality for use
		$extlinkconf['parameter'] = sprintf('http://altdwz.schachbund.de/db/spieler.html?zps=%s-%s', htmlspecialchars($mitgliedschaft[0]['zpsver']), htmlspecialchars($mitgliedschaft[0]['zpsmgl']));
		$extlinkconf['extTarget'] = '_blank';
		$wrappedSubpartContentArray['###HISTORYLINK###'] = $local_cObj->typolinkWrap($extlinkconf);

		foreach ($spieler as $key => $value) {
			$field = 'singleviewfield_' . htmlspecialchars($key);
			$text = 'singleviewtext_' . htmlspecialchars($key);
			$markContentArray['###' . strtoupper($field) . '###'] = $local_cObj->stdWrap(htmlspecialchars($value), $this->conf['singleView.'][$field . '_stdWrap.']);
			$markContentArray['###' . strtoupper($text) . '###'] = $local_cObj->stdWrap($this->pi_getLL(strtoupper($text)), $this->conf['singleView.'][$text . '_stdWrap.']);
		}

		foreach ($mitgliedschaft[0] as $key => $value) {
			$field = 'singleviewfield_' . htmlspecialchars($key);
			$text = 'singleviewtext_' . htmlspecialchars($key);
			$markContentArray['###' . strtoupper($field) . '###'] = $local_cObj->stdWrap(htmlspecialchars($value), $this->conf['singleView.'][$field . '_stdWrap.']);
			$markContentArray['###' . strtoupper($text) . '###'] = $local_cObj->stdWrap($this->pi_getLL(strtoupper($text)), $this->conf['singleView.'][$text . '_stdWrap.']);
		}


		foreach ($turnier[0] as $key => $value) {
			$text = 'singleviewtext_' . htmlspecialchars($key);
			$markContentArray['###' . strtoupper($text) . '###'] = $local_cObj->stdWrap($this->pi_getLL(strtoupper($text)), $this->conf['singleView.'][$text . '_stdWrap.']);
		}


		// loop through the array
		foreach ($turnier as $rowIndex => $line) {

				// prepare fill marker array
				if($rowIndex+1 == count($turnier)) { $line['entrynr'] = 'AKT'; }

				foreach ($line as $key => $value) {
					$field = 'singleviewfield_' . htmlspecialchars($key);
					$markerArray['###' . strtoupper($field) . '###'] = $local_cObj->stdWrap(htmlspecialchars($value), $this->conf['singleView.'][$field . '_stdWrap.']);
				}

				$isEvenRow = !($rowIndex % 2);
				$rowSubpart = $subTemplate[$isEvenRow ? 'row' : 'row_alt'];

				$subpartContentArray['###SINGLEROW###'] .= $this->cObj->substituteMarkerArrayCached($rowSubpart, $markerArray);

		}

		// Preconfigure the typolink
		$local_cObj->setCurrentVal($GLOBALS['TSFE']->id);

		// Configure typolink functionality for use
		$typolinkConf['parameter.']['current'] = 1;
		$typolinkConf['useCacheHash'] = $this->allowCaching;
		$typolinkConf['no_cache'] = !$this->allowCaching;

		// Fill 'typolinkWrap'd subpart marker
		$wrappedSubpartContentArray['###LINK###'] = $local_cObj->typolinkWrap($typolinkConf);

		//  prepare and display the graph with jqplot
		if($this->conf['jqplot'] == 1) {
			// adds the javascipt for the jqgraph to the siteheader
			$markContentArray['###SINGLEVIEW_DWZCHART###'] = '<div id="chartdiv" style="height:' . $this->conf['jqplot.']['height'] . ';width:' . $this->conf['jqplot.']['width'] . '; "></div>';
			$GLOBALS['TSFE']->setJS($this->extKey, $this->cObj->substituteMarkerArrayCached($this->getPlotJava($turnier), $markContentArray));
		} else {
			$markContentArray['###SINGLEVIEW_DWZCHART###'] = '';
		}

		// hide resulttable when there is nothing to display
		if(!$lines['turnier']) {
			$subpartContentArray['###SINGLEVIEW_RESULTTABLE###'] = '';
			$markContentArray['###SINGLEVIEW_DWZCHART###'] = '';
		}

		// Compile and return output
		return $this->cObj->substituteMarkerArrayCached($singleViewTemplate, $markContentArray, $subpartContentArray, $wrappedSubpartContentArray);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$date: ...
	 * @return	[type]		...
	 */
	protected function getLastEvaluation($date) {
		return date('W/o', strtotime($date));
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$zps: ...
	 * @return	[type]		...
	 */
	protected function getListViewData($zps) {
		$url = 'http://www.schachbund.de/php/dewis/verein.php?zps=' . htmlspecialchars($zps) . '&format=array';

		// use cachefile. refresh every 60 minutes
		$cachefile = PATH_site . 'typo3temp/dsb_dwzlist/cache_' . $zps;
		if(file_exists($cachefile) && (time() - filemtime($cachefile) < 60*60)){
			// read data from cachefile
			$urlcontent = t3lib_div::getURL($cachefile);
		} else {
			//retrieve data from url
			$urlcontent = t3lib_div::getURL($url);
			if($urlcontent){
				// if data successful fetched, write to file
				t3lib_div::writeFileToTypo3tempDir($cachefile, $urlcontent);
			} else if(file_exists($cachefile)){
				// fetching new data failed, use cached version.
				$urlcontent = t3lib_div::getURL($cachefile);
			}
		}

		// Daten als Array laden
		$array = unserialize($urlcontent);

		// determine fe render charset
		$renderCharset = $GLOBALS['TSFE']->renderCharset;

		// test if conversion from utf-8 charset is necessary
		if ($renderCharset != 'utf-8') {
			array_walk_recursive($array, 'tx_dsbdwzlist_pi1::convertToRendercharset', $renderCharset);
		}

		// Array für die Sortierung umbauen (nur relevante Spalten)
		foreach($array as $value) {
			$dwz[] = $value['dwz'];
			$dwzindex[] = $value['dwzindex'];
		}

		// Nach DWZ und Index sortieren
		array_multisort($dwz, SORT_DESC, SORT_NUMERIC, $dwzindex, SORT_DESC, SORT_NUMERIC, $array);

		return $array;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$zps: ...
	 * @return	[type]		...
	 */
	protected function listView($zps) {

		$lines = $this->getListViewData($zps);

		if (count($lines) == 0) {
			$template = $this->cObj->getSubpart($this->templateCode, '###LIST_VIEW_NO_CLUB###');
			$content .= $this->cObj->substituteMarker($template, '###TEXT_CLUB_NOT_FOUND###', $this->pi_getLL('club_not_found'));
		}
		else {
			// Get template for LIST view
			$listViewTemplate = $this->cObj->getSubpart($this->templateCode, '###LIST_VIEW###');
			$subTemplate['singlerow'] = $this->cObj->getSubpart($listViewTemplate, '###SINGLEROW###');
			$subTemplate['row'] = $this->cObj->getSubpart($subTemplate['singlerow'], '###ITEM###');
			$subTemplate['row_alt'] = $this->cObj->getSubpart($subTemplate['singlerow'], '###ITEM_ALT###');

			// prepare all the necessary objects and arrays
			$local_cObj = t3lib_div::makeInstance('tslib_cObj');

			// fill listview text markers array
			$markers = array('copyright', 'placement', 'mglnr');
			foreach ($markers as $marker) {
				$m = 'listviewtext_' . $marker;
				$markContentArray['###' . strtoupper($m) . '###'] = $local_cObj->stdWrap($this->pi_getLL(strtoupper($m)), $this->conf['listView.'][$m . '_stdWrap.']);
			}

			$markers = array_keys($lines[0]);
			$markers[] = 'evaluation';
			foreach ($markers as $marker) {
				$m = 'listviewtext_' . htmlspecialchars($marker);
				$markContentArray['###' . strtoupper($m) . '###'] = $local_cObj->stdWrap($this->pi_getLL(strtoupper($m)), $this->conf['listView.'][$m . '_stdWrap.']);
			}


			// loop through the array
			foreach ($lines as $rowIndex => $line) {

				// prepare fill marker array
				$line['placement'] = $rowIndex+1;
				$line['evaluation'] = $this->getLastEvaluation($line['turnierende']);
				foreach ($line as $key => $value) {
					$field = 'listviewfield_' . htmlspecialchars($key);
					$markerArray['###' . strtoupper($field) . '###'] = $local_cObj->stdWrap(htmlspecialchars($value), $this->conf['listView.'][$field . '_stdWrap.']);
				}

				// Preconfigure the typolink
				$local_cObj->setCurrentVal($GLOBALS['TSFE']->id);

				// Configure typolink functionality for use
				$typolinkConf['parameter.']['current'] = 1;
				$typolinkConf['additionalParams'] .= '&tx_dsbdwzlist_pi1[pkz]=' . $line['id'];
				$typolinkConf['useCacheHash'] = $this->allowCaching;
				$typolinkConf['no_cache'] = !$this->allowCaching;
				$typolinkConf['parameter.']['wrap'] = '|,'.$GLOBALS['TSFE']->type;

				// Fill 'typolinkWrap'd subpart marker
				$wrappedSubpartContentArray['###LINK###'] = $local_cObj->typolinkWrap($typolinkConf);


				$isEvenRow = !($rowIndex % 2);
				$rowSubpart = $subTemplate[$isEvenRow ? 'row' : 'row_alt'];

				$subpartContentArray['###SINGLEROW###'] .= $this->cObj->substituteMarkerArrayCached($rowSubpart, $markerArray, null, $wrappedSubpartContentArray);

			}

			$content = $this->cObj->substituteMarkerArrayCached($listViewTemplate, $markContentArray, $subpartContentArray);

		}

		// Compile and return output
		return $content;
	}



}


if (defined('TYPO3_MODE') && isset($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/dsb_dwzlist/pi1/class.tx_dsbdwzlist_pi1.php'])) {
	include_once($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/dsb_dwzlist/pi1/class.tx_dsbdwzlist_pi1.php']);
}

?>
